@@ -169,12 +169,16 @@ labels.classify <- function(labels) {
 }
 
 relevel.mnemonics <- function(labels, mnemonics = NULL) {
-  if (!is.factor(labels)) stop("Expected labels as factor")
+  if (!is.factor(labels))
+    stop("Expected labels as factor")
+
   label.map <- map.mnemonics(levels(labels), mnemonics = mnemonics)
+
   if (length(unique(label.map$labels)) != length(label.map$labels)) {
     stop(paste("Error: mnemonic replacement labels are not unique:",
                paste(label.map$labels, collapse = ", ")))
   }
+
   levels(labels) <- label.map$labels
   factor(labels, levels(labels)[label.map$order])
 }
