@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 196 $"
+__version__ = "$Revision: 228 $"
 
 """
 Evaluates the overlap between two BED files, based upon the spec at:
@@ -11,25 +11,22 @@ Author: Orion Buske <stasis@uw.edu>
 Date:   August 18, 2009
 """
 
-import math
 import os
 import sys
 
 from collections import defaultdict
-from math import ceil
 from numpy import bincount, cast, iinfo, invert, logical_or, zeros
-from rpy2.robjects import r, numpy2ri
 
-from . import Segmentation
-from .common import check_clobber, die, get_ordered_labels, image_saver, load_features, make_tabfilename, map_mnemonics, r_source, setup_directory, SUFFIX_GZ, tab_saver
+from . import Annotations, log, Segmentation
+from .common import check_clobber, die, get_ordered_labels, make_tabfilename, r_plot, r_source, setup_directory, SUFFIX_GZ, tab_saver
 from .html import save_html_div
 
 # A package-unique, descriptive module name used for filenames, etc
 MODULE = "overlap"
 
 NAMEBASE = "%s" % MODULE
-HEATMAP_NAMEBASE = os.extsep.join([NAMEBASE, "heatmap"])
-SIGNIFICANCE_NAMEBASE = os.extsep.join([NAMEBASE, "significance"])
+PERFORMANCE_NAMEBASE = os.extsep.join([NAMEBASE, "performance"])
+
 OVERLAPPING_SEGMENTS_NAMEBASE = os.extsep.join([NAMEBASE, "segments"])
 OVERLAPPING_SEGMENTS_FIELDS = ["chrom", "start (zero-indexed)",
                                "end (exclusive)", "group",
@@ -77,15 +74,14 @@ def calc_overlap(subseg, qryseg, quick=False, clobber=False, mode=MODE_DEFAULT,
             outfilepath = os.path.join(dirpath, outfilename)
             check_clobber(outfilepath, clobber=clobber)
             outfiles[sub_label_key] = open(outfilepath, "w")
-            print >>outfiles[sub_label_key], header
+            outfiles[sub_label_key].write("%s\n" % header)
 
     counts = zeros((len(sub_labels), len(qry_labels)), dtype="int")
     totals = zeros(len(sub_labels), dtype="int")
     nones = zeros(len(sub_labels), dtype="int")
 
     for chrom in subseg.chromosomes:
-        if verbose:
-            print >>sys.stderr, "\t%s" % chrom
+        log("\t%s" % chrom, verbose)
 
         try:
             qry_segments = qryseg.chromosomes[chrom]
@@ -192,7 +188,7 @@ def calc_overlap(subseg, qryseg, quick=False, clobber=False, mode=MODE_DEFAULT,
                     except IndexError:
                         pass
                     values = [str(val) for val in values]
-                    print >>outfiles[sublabelkey], "\t".join(values)
+                    outfiles[sublabelkey].write("%s\n" % "\t".join(values))
 
             # Organize overlapping_segments by qrylabelkey
             label_overlaps = defaultdict(list)  # Per qrylabelkey
@@ -242,14 +238,14 @@ def make_tab_row(col_indices, data, none, total):
 
 ## Saves the data to a tab file
 def save_tab(dirpath, row_labels, col_labels, counts, totals, nones, mode,
-             namebase=NAMEBASE, clobber=False):
+             namebase=NAMEBASE, clobber=False, verbose=True):
     assert counts is not None and totals is not None and nones is not None
 
     row_label_keys, row_labels = get_ordered_labels(row_labels)
     col_label_keys, col_labels = get_ordered_labels(col_labels)
     colnames = [col_labels[label_key] for label_key in col_label_keys]
     metadata = {"mode": mode}
-    with tab_saver(dirpath, namebase, clobber=clobber,
+    with tab_saver(dirpath, namebase, clobber=clobber, verbose=verbose,
                    metadata=metadata) as count_saver:
         header = [""] + colnames + [NONE_COL, TOTAL_COL]
         count_saver.writerow(header)
@@ -259,54 +255,47 @@ def save_tab(dirpath, row_labels, col_labels, counts, totals, nones, mode,
             row.insert(0, row_labels[row_label_key])
             count_saver.writerow(row)
 
-def save_plot(dirpath, namebase=NAMEBASE, clobber=False,
-              row_mnemonic_file="", col_mnemonic_file=""):
+def save_plot(dirpath, namebase=NAMEBASE, clobber=False, verbose=True,
+              row_mnemonic_file=None, col_mnemonic_file=None,
+              cluster=False):
     start_R()
 
     tabfilename = make_tabfilename(dirpath, namebase)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    if not row_mnemonic_file:
-        row_mnemonic_file=""
-    if not col_mnemonic_file:
-        col_mnemonic_file=""
+    r_plot("save.overlap.heatmap", dirpath, namebase, tabfilename,
+           mnemonic_file=row_mnemonic_file,
+           col_mnemonic_file=col_mnemonic_file,
+           clobber=clobber, cluster=cluster, verbose=verbose)
 
-    r["save.overlap"](dirpath, namebase, tabfilename,
-                      mnemonic_file=row_mnemonic_file,
-                      col_mnemonic_file=col_mnemonic_file,
-                      clobber=clobber)
-
-def save_heatmap_plot(dirpath, namebase=NAMEBASE, clobber=False,
-                      row_mnemonic_file="", col_mnemonic_file="",
-                      cluster=False):
+def save_performance_plot(dirpath, namebase=PERFORMANCE_NAMEBASE,
+                          clobber=False, verbose=True,
+                          row_mnemonic_file=None, col_mnemonic_file=None):
     start_R()
 
-    tabfilename = make_tabfilename(dirpath, namebase)
+    tabfilename = make_tabfilename(dirpath, NAMEBASE)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
-    if not row_mnemonic_file:
-        row_mnemonic_file=""
-    if not col_mnemonic_file:
-        col_mnemonic_file=""
+    r_plot("save.overlap.performance", dirpath, namebase, tabfilename,
+           mnemonic_file=row_mnemonic_file,
+           col_mnemonic_file=col_mnemonic_file,
+           clobber=clobber, verbose=verbose)
 
-    r["save.overlap.heatmap"](dirpath, HEATMAP_NAMEBASE, tabfilename,
-                              mnemonic_file=row_mnemonic_file,
-                              col_mnemonic_file=col_mnemonic_file,
-                              clobber=clobber, cluster=cluster)
-
-def save_html(dirpath, bedfilename, featurefilename, mode, clobber=False):
+def save_html(dirpath, bedfilename, featurefilename, mode,
+              mnemonicfile=None, clobber=False):
     bedfilename = os.path.basename(bedfilename)
     featurebasename = os.path.basename(featurefilename)
-    extra_namebases = {"heatmap": HEATMAP_NAMEBASE}
+    extra_namebases = {"performance": PERFORMANCE_NAMEBASE}
 
     title = "%s (%s)" % (HTML_TITLE_BASE, featurebasename)
 
     significance = ""
     save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
-                  title=title, tablenamebase=NAMEBASE,
-                  extra_namebases = extra_namebases,
+                  title=title, tables={"":(PERFORMANCE_NAMEBASE, "exact")},
+                  mnemonicfile=mnemonicfile,
+                  extra_namebases=extra_namebases,
                   module=MODULE, by=mode, significance=significance,
                   bedfilename=bedfilename, featurefilename=featurebasename)
 
@@ -333,29 +322,14 @@ def overlap(bedfilename, featurefilename, dirpath, regionfilename=None,
     if not replot:
         setup_directory(dirpath)
 
-        segmentation = Segmentation(bedfilename)
-
-        if is_file_type(featurefilename, 'bed'):
-            features = Segmentation(featurefilename, verbose=verbose)
-        elif is_file_type(featurefilename, 'gff'):
-            features = load_features(featurefilename, verbose=verbose)
-        elif is_file_type(featurefilename, 'gtf'):
-            features = load_features(featurefilename, gtf=True,
-                                     sort=True, verbose=verbose)
-        else:
-            raise NotImplementedError("Only bed, gff, and gtf files are \
-supported for FEATUREFILE. If the file is in one of these formats, please \
-use the appropriate extension")
+        segmentation = Segmentation(bedfilename, verbose=verbose)
+        features = Annotations(featurefilename, verbose=verbose)
 
         seg_labels = segmentation.labels
         feature_labels = features.labels
-        mnemonics = map_mnemonics(seg_labels, mnemonic_filename)
-        feature_mnemonics = map_mnemonics(feature_labels,
-                                          feature_mnemonic_filename)
 
         # Overlap of segmentation with features
-        if verbose:
-            print >>sys.stderr, "Measuring overlap..."
+        log("Measuring overlap:", verbose)
 
         counts, nones, totals = \
             calc_overlap(segmentation, features, clobber=clobber,
@@ -364,35 +338,31 @@ use the appropriate extension")
                          quick=quick, dirpath=dirpath, verbose=verbose)
 
         save_tab(dirpath, seg_labels, feature_labels,
-                 counts, nones, totals, mode=mode, clobber=clobber)
+                 counts, nones, totals, mode=mode,
+                 clobber=clobber, verbose=verbose)
 
     if not noplot:
-        save_plot(dirpath, clobber=clobber,
+        save_performance_plot(dirpath, clobber=clobber, verbose=verbose,
+                              row_mnemonic_file=mnemonic_filename,
+                              col_mnemonic_file=feature_mnemonic_filename)
+        save_plot(dirpath, clobber=clobber, cluster=cluster, verbose=verbose,
                   row_mnemonic_file=mnemonic_filename,
                   col_mnemonic_file=feature_mnemonic_filename)
-        save_heatmap_plot(dirpath, clobber=clobber,
-                          row_mnemonic_file=mnemonic_filename,
-                          col_mnemonic_file=feature_mnemonic_filename,
-                          cluster=cluster)
 
-    if verbose:
-        print >>sys.stderr, "Saving HTML div...",
-
-    sys.stdout.flush()  # Necessary to make sure html errors don't clobber print
-    save_html(dirpath, bedfilename, featurefilename, mode=mode, clobber=clobber)
-    if verbose:
-        print >>sys.stderr, "done"
+    log("Saving HTML div...", verbose)
+    save_html(dirpath, bedfilename, featurefilename, mode=mode,
+              mnemonicfile=mnemonic_filename, clobber=clobber)
+    log(" done", verbose)
 
 def parse_options(args):
     from optparse import OptionParser, OptionGroup
 
-    usage = "%prog [OPTIONS] BEDFILE FEATUREFILE"
-    description = "BEDFILE must be in BED4+ format (name column used as \
-grouping variable). FEATUREFILE should be in BED4+ format or GFF format \
-(feature column used as grouping variable). Results summarize the overlap \
-of BEDFILE groups with FEATUREFILE groups. The symmetric analysis can \
-be performed (if both input files are in BED4+ format) \
-by rerunning the program with the input file arguments swapped \
+    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATIONS"
+    description = "SEGMENTATION and ANNOTATIONS files should be in BED, GFF, \
+or GTF format (grouped on 'name'/'feature' columns). \
+Results summarize the overlap \
+of SEGMENTATION groups with ANNOTATIONS groups. The symmetric analysis can \
+be performed by rerunning the program with the input file arguments swapped \
 (and a different output directory). A rough specification can be found here: \
 http://encodewiki.ucsc.edu/EncodeDCC/index.php/\
 Overlap_analysis_tool_specification"
@@ -406,12 +376,12 @@ Overlap_analysis_tool_specification"
                      dest="clobber", default=False,
                      help="Overwrite existing output files if the specified"
                      " directory already exists.")
+    group.add_option("-q", "--quiet", action="store_false",
+                     dest="verbose", default=True,
+                     help="Do not print diagnostic messages.")
     group.add_option("--quick", action="store_true",
                      dest="quick", default=False,
                      help="Compute values only for one chromosome.")
-    group.add_option("-v", "--verbose", action="store_true",
-                     dest="verbose", default=False,
-                     help="Print status and diagnostic messages.")
     group.add_option("--replot", action="store_true",
                      dest="replot", default=False,
                      help="Load data from output tab files and"
@@ -425,15 +395,15 @@ Overlap_analysis_tool_specification"
     group.add_option("-p", "--print-segments", action="store_true",
                      dest="print_segments", default=False,
                      help=("For each group"
-                     " in the BEDFILE, a separate output file will be"
+                     " in the SEGMENTATION, a separate output file will be"
                      " created that contains a list of all the segments that"
                      " the group was found to overlap with. Output files"
                      " are named %s.X.txt, where X is the name"
-                     " of the BEDFILE group.") % OVERLAPPING_SEGMENTS_NAMEBASE)
+                     " of the SEGMENTATION group.") % OVERLAPPING_SEGMENTS_NAMEBASE)
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Parameters")
-    group.add_option("-b", "--by", choices=MODE_CHOICES,
+    group.add_option("-b", "--by", choices=MODE_CHOICES, metavar="MODE",
                      dest="mode", type="choice", default=MODE_DEFAULT,
                      help="One of: "+str(MODE_CHOICES)+", which determines the"
                      " definition of overlap. @segments: The value"
@@ -447,8 +417,8 @@ Overlap_analysis_tool_specification"
 #                      help="For the specified file (1, 2, or both), use only"
 #                      "the midpoint of each feature instead of the entire"
 #                      " width.")
-    group.add_option("-m", "--min-overlap", type="int",
-                     dest="min_overlap", default=1,
+    group.add_option("--min-overlap", type="int",
+                     dest="min_overlap", default=1, metavar="N",
                      help="The minimum number of base pairs that two"
                      " features must overlap for them to be classified as"
                      " overlapping. This integer can be either positive"
@@ -460,16 +430,16 @@ Overlap_analysis_tool_specification"
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Files")
-    group.add_option("--mnemonic-file", dest="mnemonic_filename",
-                     default=None,
-                     help="If specified, BEDFILE groups will be shown using"
-                     " mnemonics found in this file")
-    group.add_option("--feature-mnemonic-file",
+    group.add_option("-m", "--mnemonic-file", dest="mnemonic_filename",
+                     default=None, metavar="FILE",
+                     help="If specified, SEGMENTATION groups will be shown"
+                     " using mnemonics found in FILE.")
+    group.add_option("--feature-mnemonic-file", metavar="FILE",
                      dest="feature_mnemonic_filename", default=None,
-                     help="If specified, FEATUREFILE groups will be shown"
-                     " using mnemonics found in this file")
+                     help="If specified, ANNOTATIONS groups will be shown"
+                     " using mnemonics found in FILE.")
     group.add_option("-o", "--outdir",
-                     dest="outdir", default="%s" % MODULE,
+                     dest="outdir", default="%s" % MODULE, metavar="DIR",
                      help="File output directory (will be created"
                      " if it does not exist) [default: %default]")
     parser.add_option_group(group)
