@@ -2,8 +2,6 @@
 from __future__ import division, with_statement
 
 """
-validate.py:
-
 Provides command-line and package entry points for analyzing the signal
 distribution over tracks and labels.
 
@@ -13,7 +11,7 @@ distribution over tracks and labels.
 # Must be the same as the folder containing this script
 MODULE="signal_distribution"
 
-__version__ = "$Revision: 192 $"
+__version__ = "$Revision: 228 $"
 
 
 import os
@@ -23,11 +21,10 @@ from collections import defaultdict
 from genomedata import Genome
 from itertools import repeat
 from functools import partial
-from numpy import array, ceil, floor, fromiter, histogram, isfinite, NAN, nanmax, nanmin, nansum, NINF, PINF
-from rpy2.robjects import r, numpy2ri
+from numpy import array, ceil, compress, floor, fromiter, histogram, isfinite, NAN, nanmax, nanmin, nansum, NINF, PINF
 
-from . import Segmentation
-from .common import die, image_saver, iter_segments_continuous, make_tabfilename, map_mnemonics, r_source, setup_directory, tab_reader, tab_saver
+from . import log, Segmentation
+from .common import die, iter_segments_continuous, iter_supercontig_segments, make_tabfilename, r_plot, r_source, setup_directory, tab_reader, tab_saver
 from .html import save_html_div
 from .mnemonics import create_mnemonic_file
 
@@ -40,13 +37,6 @@ NAMEBASE_STATS = os.extsep.join([NAMEBASE, "stats"])
 HTML_TITLE = "Signal value distribution"
 HTML_TEMPLATE_FILENAME = "signal_div.tmpl"
 
-PNG_WIDTH_PER_SUBPLOT = 200
-PNG_HEIGHT_PER_SUBPLOT = 150
-PNG_BASE_HEIGHT = 200
-PNG_BASE_WIDTH = 100
-
-STATS_PNG_SIZE = 1000
-
 NBINS = 100
 
 class SignalHistogram(object):
@@ -64,7 +54,7 @@ class SignalHistogram(object):
 
     @staticmethod
     def calculate(genome, segmentation, nbins=None, chroms=None,
-                  group_labels=False, calc_ranges=False,
+                  calc_ranges=False,
                   value_range=(None, None), quick=False, verbose=True):
         """Computes a set of histograms, one for each track-label pair.
 
@@ -76,11 +66,6 @@ class SignalHistogram(object):
         - a sequence of chromosome names: only those chromosomes are processed
         - False, [], None: all chromosomes are processed
 
-        if group_labels is:
-        - True: Group together all labels and only generate one histogram
-          for the track over all labels
-        - False: Generate a histogram for every track-label pair
-
         if calc_ranges is:
         - False: use the precomputed ranges stored for the entire genome file
         - True: calculate the segmentation ranges across the entire input
@@ -95,14 +80,11 @@ class SignalHistogram(object):
 
         returns a dict
 
-        if group_labels:
+        key: label (not label_key)
+        val: dict
           key: trackname
           val: [hist, edges]
-        else:
-          key: label (not label_key)
-          val: dict
-               key: trackname
-               val: [hist, edges]
+
         """
         assert genome is not None
         assert segmentation is not None
@@ -135,9 +117,6 @@ class SignalHistogram(object):
         #   Returns a new partial object which when called will behave like
         #   func called with args and keywords.
 
-        #if group_labels:  # Pretend there is just one "all" label
-        #    labels = dict([(-1, "all")])
-
         # Result dictionary: see function docstring for specification
         # Now for every trackname and for every label
         #   there's a histogram array of the right boundary and frequency
@@ -148,8 +127,7 @@ class SignalHistogram(object):
                        track_hist_funcs.iteritems())
 
         nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
-        if verbose:
-            print >>sys.stderr, "Generating signal distribution histograms"
+        log("Generating signal distribution histograms", verbose)
 
         with genome:
             if chroms:
@@ -194,18 +172,110 @@ class SignalHistogram(object):
 
                 if quick: break  # 1 chromosome
 
-        if verbose:
-            print >>sys.stderr, "Segmentation tracks: %s\n" % \
-                str(segmentation.tracks)
-            print >>sys.stderr, ("Read %s non-NaN values from"
-                                 " segmentation tracks" % nseg_dps)
+        log("Segmentation tracks: %s\n" % str(segmentation.tracks), verbose)
+        log("Read %s non-NaN values from segmentation tracks" % nseg_dps,
+            verbose)
 
         return SignalHistogram(res, nseg_dps=nseg_dps)
 
     @staticmethod
-    def read(dirpath, basename=NAMEBASE, **kwargs):
+    def calculate2(genome, segmentation, nbins=None, chroms=None,
+                  calc_ranges=False,
+                  value_range=(None, None), quick=False, verbose=True):
+        assert genome is not None
+        assert segmentation is not None
+
+        labels = segmentation.labels
+
+        label_bins = xrange(0, len(labels) + 1)  # [0, 1, ..., n, n + 1]
+
+        tracks = genome.tracknames_continuous
+        max_bins = ceil(genome.maxs).astype(int)
+        min_bins = floor(genome.mins).astype(int)
+        # A dict from tracks to a range tuple
+        track_ranges = dict(zip(tracks, zip(min_bins, max_bins)))
+
+        if len(tracks) == 0:
+            die("Trying to calculate histogram for no tracks")
+
+        # key: trackname
+        # val: track_hist_func: a function that generates a histogram from data
+        #                       with uniform bins within a track
+        # For every trackname, there's a function that generates histograms
+        track_hist_funcs = dict((trackname,
+                                 partial(histogram,
+                                         bins=xrange(min_bin, max_bin + 1),
+                                         new=True))
+                                for trackname, (min_bin, max_bin) in
+                                track_ranges.iteritems())
+        # Trackname is one the track names, track_range is (min,max)
+        # functools.partial(func[, *args][, **keywords])
+        #   Returns a new partial object which when called will behave like
+        #   func called with args and keywords.
+
+        # Result dictionary: see function docstring for specification
+        # Now for every trackname and for every label
+        #   there's a histogram array of the right boundary and frequency
+        res = dict((trackname, dict((label,
+                                     list(track_hist_func(array([]))))
+                                    for label in labels.itervalues()))
+                   for trackname, track_hist_func in \
+                       track_hist_funcs.iteritems())
+
+        nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
+        log("Generating signal distribution histograms", verbose)
+
+        with genome:
+            if chroms:
+                chromosomes = [genome[chrom] for chrom in chroms]
+            else:
+                chromosomes = genome
+
+            for chromosome in chromosomes:
+                # Iterate through supercontigs and segments together
+                for supercontig, segments in \
+                        iter_supercontig_segments(chromosome, segmentation,
+                                                  verbose=verbose):
+                    continuous = supercontig.continuous
+                    for key in labels:
+                        seg_label = labels[key]
+                        condition = segments['key'] == key
+                        print "starting==========="
+                        continuous_chunk = compress(condition, continuous,
+                                                    axis=0)
+                        print "done==========="
+                        sys.exit(1)
+                        #sums = nansum(continuous_chunk, axis=0)
+                        #sum_sqs = nansum(power(continuous_chunk, 2.0), axis = 0)
+                        for trackname, track_hist_func \
+                                in track_hist_funcs.iteritems():
+                            # col_index is the index of the trackname
+                            #   (if 5 tracks, it can be 0, 1, 2, 3 or 4)
+                            col_index = chromosome.index_continuous(trackname)
+
+                            cur_col = continuous_chunk[:, col_index]
+                            cur_col_nonan = cur_col[isfinite(cur_col)]
+                            # Edges is a list with the left edge of every bin,
+                            #   hist is the frequency for every bin
+                            # This calls numpy.histogram
+                            hist, edges = track_hist_func(cur_col_nonan)
+
+                            cur_res = res[trackname][seg_label]
+                            cur_res[0] += hist
+
+                if quick: break  # 1 chromosome
+
+        log("Segmentation tracks: %s\n" % str(segmentation.tracks), verbose)
+        log("Read %s non-NaN values from segmentation tracks" % nseg_dps,
+            verbose)
+
+        return SignalHistogram(res, nseg_dps=nseg_dps)
+
+
+    @staticmethod
+    def read(dirpath, namebase=NAMEBASE, **kwargs):
         """Read a histogram from an output directory"""
-        with tab_reader(dirpath, basename, **kwargs) as (reader, metadata):
+        with tab_reader(dirpath, namebase, **kwargs) as (reader, metadata):
             histogram = {}
             try:
                 nseg_dps = int(metadata["nseg_dps"])
@@ -228,9 +298,9 @@ class SignalHistogram(object):
 
         return SignalHistogram(histogram, nseg_dps=nseg_dps)
 
-    def save(self, dirpath, basename=NAMEBASE, **kwargs):
+    def save(self, dirpath, namebase=NAMEBASE, **kwargs):
         """Saves the histogram data to a tab file"""
-        with tab_saver(dirpath, basename, metadata=self.metadata,
+        with tab_saver(dirpath, namebase, metadata=self.metadata,
                        **kwargs) as saver:
             for trackname in sorted(self.data.keys()):
                 label_hists = self.data[trackname]
@@ -288,9 +358,10 @@ def calc_stats(histogram):
 
 
 ## Saves the track stats to a tab file
-def save_stats_tab(stats, dirpath, clobber=False,
-                   basename=NAMEBASE_STATS, fieldnames=FIELDNAMES_STATS):
-    with tab_saver(dirpath, basename, fieldnames, clobber=clobber) as saver:
+def save_stats_tab(stats, dirpath, clobber=False, verbose=True,
+                   namebase=NAMEBASE_STATS, fieldnames=FIELDNAMES_STATS):
+    with tab_saver(dirpath, namebase, fieldnames, verbose=verbose,
+                   clobber=clobber) as saver:
         for label, label_stats in stats.iteritems():
             for trackname, track_stats in label_stats.iteritems():
                 mean = track_stats["mean"]
@@ -350,7 +421,7 @@ def load_track_ranges(genome, segmentation=None):
     res = defaultdict(constant_factory((PINF, NINF)))
     for chromosome in genome:
         if segmentation is not None:
-            print >>sys.stderr, "\t%s" % chromosome.name
+            log("\t%s" % chromosome.name)
             res_chrom = seg_min_max(chromosome, segmentation)
             for trackname, limits in res_chrom.iteritems():
                 old_min, old_max = res[trackname]
@@ -371,171 +442,10 @@ def load_track_ranges(genome, segmentation=None):
     # Cast out of defaultdict
     return dict(res)
 
-# ## Computes a set of histograms, one for each track-label pair.
-# def calc_histogram(genome, segmentation, nbins=None,
-#                    group_labels=False, calc_ranges=False,
-#                    value_range=(None, None), quick=False):
-#     """
-#     if nbins is:
-#     - None: There will be a bin for every possible value
-#     - > 0: There will be this many bins
-
-#     if group_labels is:
-#     - True: Group together all labels and only generate one histogram
-#       for the track over all labels
-#     - False: Generate a histogram for every track-label pair
-
-#     if calc_ranges is:
-#     - False: use the precomputed ranges stored for the entire genome file
-#     - True: calculate the segmentation ranges across the entire input
-
-#     value_range is a (min, max) tuple:
-#       if either value is None, it is ignored
-#       if min/max is not None, it is used as a limit for all binning
-#       if both are specified, calc_ranges is ignored
-
-#     if quick is:
-#     - True: the histogram is calculated for only the first chromosome
-
-#     returns a dict
-
-#     if group_labels:
-#       key: trackname
-#       val: [hist, edges]
-#     else:
-#       key: label_key
-#       val: dict
-#            key: trackname
-#            val: [hist, edges]
-#     """
-#     assert genome is not None
-#     assert segmentation is not None
-
-#     if calc_ranges and any(lim is None for lim in value_range):
-#         # Calculate ranges over segmentation only
-#         print >>sys.stderr, "Calculating track ranges...",
-#         track_ranges = load_track_ranges(genome, segmentation)
-#         print >>sys.stderr, "done."
-#     else:
-#         # Lookup ranges in genomedata
-#         track_ranges = load_track_ranges(genome)
-
-#     # Override ranges as necessary
-#     value_min, value_max = value_range
-#     for trackname, track_range in track_ranges.iteritems():
-#         track_min, track_max = track_range
-#         if value_min is not None:
-#             track_min = value_min
-#         if value_max is not None:
-#             track_max = value_max
-#         track_ranges[trackname] = (track_min, track_max)
-
-#     tracks = track_ranges.keys()  # Re-select just the tracks used
-
-#     if len(tracks) == 0:
-#         die("Trying to calculate histogram for no tracks")
-
-#     # Calculate bins for each track
-#     if nbins is None:  # Have a bin for every possible integer value
-#         track_bins = dict([(trackname,
-#                             math.ceil(track_range[1]) -
-#                             math.floor(track_range[0]))
-#                            for trackname, track_range in track_ranges])
-#     else:  # Use the given number of bins
-#         assert nbins > 0
-#         track_bins = dict([(trackname, nbins) for trackname in tracks])
-
-#     # key: trackname
-#     # val: histogram_func: a function that generates a histogram from data
-#     #                      with uniform bins within a track
-#     # For every trackname, there's a function that generates histograms
-#     histogram_funcs = dict((trackname, partial(histogram,
-#                                                bins=track_bins[trackname],
-#                                                range=track_ranges[trackname],
-#                                                new=True))
-#                            for trackname in tracks)
-#     # Trackname is one the track names, track_range is (min,max)
-#     # functools.partial(func[, *args][, **keywords])
-#     #   Returns a new partial object which when called will behave like func
-#     #   called with args and keywords.
-
-#     labels = segmentation.labels
-#     if group_labels:  # Pretend there is just one "all" label
-#         labels = dict([(-1, "all")])
-
-#     # Return dictionary: see function docstring for specification
-#     # Now for every label and for every trackname
-#     #   there's a histogram array of the right boundary and frequency
-#     res = dict((label_key, dict((trackname,
-#                                  list(histogram_func(array([]))))
-#                                 for trackname, histogram_func
-#                                 in histogram_funcs.iteritems()))
-#                for label_key in labels)
-
-#     print >>sys.stderr, "Generating signal distribution histograms"
-#     ntracks = len(tracks)
-#     nlabels = len(labels)
-#     nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
-
-#     with genome:
-#         for chromosome in genome:
-#             chrom = chromosome.name
-
-#             # Iterate through supercontigs and segments together
-#             for segment, continuous_seg in \
-#                     iter_segments_continuous(chromosome, segmentation):
-
-#                 if group_labels:
-#                     cur_res = res  # All labels are used
-#                 else:
-#                     seg_label_key = segment['key']
-#                     cur_res = res[seg_label_key]
-
-#                 # Iterate through each track
-#                 for trackname, histogram_func in histogram_funcs.iteritems():
-#                     # col_index is the index of the trackname
-#                     #   (if 5 tracks, it can be 0, 1, 2, 3 or 4)
-#                     col_index = chromosome.index_continuous(trackname)
-
-#                     # continuous_col is the "intensity"
-#                     #   (continuous number) from the data track
-#                     # len(supercontig_map) = len(continuous_col)
-#                     #   because they are for the same segment
-#                     continuous_col = continuous_seg[:, col_index]
-
-#                     # Remove the NaN's, otherwise the numpy.histogram function
-#                     #   gets confused and misses some of the data
-#                     cur_col_nonan = remove_nans(continuous_col)
-
-#                     # Keep track of number of real data values processed
-#                     if trackname in segmentation.tracks:
-#                         nseg_dps += cur_col_nonan.shape[0]
-
-#                     # Edges is a list containing the left edge of every bin,
-#                     #   hist is the frequency for every bin
-#                     # This calls numpy.histogram
-#                     hist, edges = histogram_func(cur_col_nonan)
-
-#                     res_trackname = cur_res[trackname]
-
-#                     assert (res_trackname[1] == edges).all()
-#                     res_trackname[0] += hist
-#             if quick: break  # 1 chromosome
-
-#     print ""
-#     print "Segmentation tracks: %s" % str(segmentation.tracks)
-#     print "Read %s non-NaN values from segmentation tracks\n" % nseg_dps
-#     return res, nseg_dps
-
-
-
-
-
-
 
 # ## Computes a set of histograms, one for each track-label pair.
 # def calc_histogram2(genome, segmentation, quick=False, verbose=True,
-#                    group_labels=False, calc_ranges=False,
+#                    calc_ranges=False,
 #                    value_range=(None, None), nbins=None,):
 #     """
 #     if quick is:
@@ -678,21 +588,20 @@ def load_track_ranges(genome, segmentation=None):
 #     return stats
 
 # ## Saves the histogram data to a tab file
-# def save_tab(labels, histogram, dirpath, clobber=False, group_labels=False,
-#              basename=NAMEBASE, fieldnames=FIELDNAMES):
-#     with tab_saver(dirpath, basename, fieldnames, clobber=clobber) as saver:
+# def save_tab(labels, histogram, dirpath, clobber=False,
+#              namebase=NAMEBASE, fieldnames=FIELDNAMES):
+#     with tab_saver(dirpath, namebase, fieldnames, clobber=clobber) as saver:
 #         for label_key, label_histogram in histogram.iteritems():
 #             label = labels[label_key]
 #             for trackname, (hist, edges) in label_histogram.iteritems():
 #                 for lower_edge, count in zip(edges, hist.tolist() + ["NA"]):
 #                     saver.writerow(locals())
-#             if group_labels: return  # Only write grouped label
 
 
 # ## Saves the track stats to a tab file
-# def save_stats_tab2(labels, stats, dirpath, clobber=False, group_labels=False,
-#                    basename=NAMEBASE_STATS, fieldnames=FIELDNAMES_STATS):
-#     with tab_saver(dirpath, basename, fieldnames, clobber=clobber) as saver:
+# def save_stats_tab2(labels, stats, dirpath, clobber=False,
+#                    namebase=NAMEBASE_STATS, fieldnames=FIELDNAMES_STATS):
+#     with tab_saver(dirpath, namebase, fieldnames, clobber=clobber) as saver:
 #         for trackname, track_stats in stats.iteritems():
 #             for label_key, label_stats in track_stats.iteritems():
 #                 label = labels[label_key]
@@ -700,50 +609,11 @@ def load_track_ranges(genome, segmentation=None):
 #                 sd = label_stats["sd"]
 #                 saver.writerow(locals())
 
-## Plots the tab file data and saves the plots
-def save_plot(ntracks, nlabels, segtracks, dirpath,
-              clobber=False, group_labels=False,
-              basename=NAMEBASE, ecdf=False, mnemonics=[]):
+def save_stats_plot(dirpath, namebase=NAMEBASE_STATS, filename=None,
+                    clobber=False, mnemonic_file=None, translation_file=None,
+                    allow_regex=False, gmtk=False, verbose=True):
     """
-    ntracks: number of tracks in the Genome
-    nlabels: number of labels in the Segmentation
-    segtracks: a list of strings; the name of each track in the segmentation
-    """
-    start_R()
-
-    if group_labels:  # Only one plot per track
-        nlabels = 0
-        mnemonics = []  # Ignore mnemonics
-
-    # load data from the file just written out
-    # it's bad to remake the filename
-    # XXX: rewriting everything in terms of a class would allow avoiding this
-    tabfilename = make_tabfilename(dirpath, basename)
-    if not os.path.isfile(tabfilename):
-        die("Unable to find tab file: %s" % tabfilename)
-
-    print >>sys.stderr, "\tPlotting distributions from: %s" % tabfilename
-    if ecdf:
-        mode = "ecdf"
-    else:
-        mode = "histogram"
-
-    # XXX: Figure out ntracks and nlabels from file
-    with image_saver(dirpath, basename,
-                     width=PNG_BASE_WIDTH + \
-                         ntracks * PNG_WIDTH_PER_SUBPLOT,
-                     height=PNG_BASE_HEIGHT + \
-                         (nlabels + 1) * PNG_HEIGHT_PER_SUBPLOT,
-                     clobber=clobber,
-                     downsample=True):  # Speeds up plotting
-        r.plot(r["plot.signal"](tabfilename, segtracks=segtracks,
-                                mnemonics=mnemonics, mode=mode))
-
-def save_stats_plot(dirpath, basename=NAMEBASE_STATS, datafilename=None,
-                    clobber=False, mnemonics=[], translation_file=None,
-                    gmtk=False):
-    """
-    if datafilename is specified, it overrides dirpath/basename.tab as
+    if filename is specified, it overrides dirpath/namebase.tab as
     the data file for plotting.
 
     """
@@ -751,22 +621,17 @@ def save_stats_plot(dirpath, basename=NAMEBASE_STATS, datafilename=None,
     start_R()
     r_source("track_statistics.R")
 
-    if datafilename is None:
-        datafilename = make_tabfilename(dirpath, basename)
+    if filename is None:
+        filename = make_tabfilename(dirpath, namebase)
 
-    if not os.path.isfile(datafilename):
-        die("Unable to find stats data file: %s" % datafilename)
+    if not os.path.isfile(filename):
+        die("Unable to find stats data file: %s" % filename)
 
-    if translation_file is None:
-        translation_file = ""  # None is not currently supported in rpy2
-
-    with image_saver(dirpath, basename,
-                     width=STATS_PNG_SIZE,
-                     height=STATS_PNG_SIZE,
-                     clobber=clobber):
-        r.plot(r["plot.track.stats"](datafilename, mnemonics=mnemonics,
-                                     translation_file=translation_file,
-                                     gmtk=gmtk))
+    r_plot("save.track.stats", dirpath, namebase, filename,
+           mnemonic_file=mnemonic_file,
+           translation_file=translation_file,
+           as_regex=allow_regex, gmtk=gmtk,
+           clobber=clobber, verbose=verbose)
 
 def save_html(dirpath, genomedatadir, nseg_dps=None,
               ecdf=False, clobber=False):
@@ -786,26 +651,22 @@ def save_html(dirpath, genomedatadir, nseg_dps=None,
                   extra_namebases=extra_namebases)
 
 ## Package entry point
-def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
+def validate(bedfilename, genomedatadir, dirpath,
              clobber=False, calc_ranges=False, inputdirs=None,
-             quick=False, replot=False, noplot=False,
+             quick=False, replot=False, noplot=False, verbose=True,
              nbins=NBINS, value_range=(None, None),
-             ecdf=False, mnemonicfilename=None,
+             ecdf=False, mnemonic_file=None,
              create_mnemonics=False, chroms=None):
 
-    setup_directory(dirpath)
-    genome = Genome(genomedatadir)
-    segmentation = Segmentation(bedfilename)
+    if not replot:
+        setup_directory(dirpath)
+        genome = Genome(genomedatadir)
+        segmentation = Segmentation(bedfilename, verbose=verbose)
 
-    ntracks = genome.num_tracks_continuous  # All tracks; not just segtracks
-    segtracks = segmentation.tracks
-    labels = segmentation.labels
+        ntracks = genome.num_tracks_continuous  # All tracks; not just segtracks
+        segtracks = segmentation.tracks
+        labels = segmentation.labels
 
-    if group_labels:
-        nbins = None  # Set bins automatically
-        nlabels = 0  # Not using labels
-        fieldnames = FIELDNAMES[1:]  # No 'label' field
-    else:
         nlabels = len(labels)
         fieldnames = FIELDNAMES
 
@@ -813,43 +674,38 @@ def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
         histogram = SignalHistogram()
         for inputdir in inputdirs:
             try:
-                sub_histogram = SignalHistogram.read(inputdir)
+                sub_histogram = SignalHistogram.read(inputdir, verbose=verbose)
             except IOError, e:
-                print >>sys.stderr, "Problem reading data from %s: %s" % \
-                    (inputdir, e)
+                log("Problem reading data from %s: %s" % (inputdir, e))
             else:
                 histogram.add(sub_histogram)
 
     elif not replot:
         # Generate histogram
         histogram = SignalHistogram.calculate(genome, segmentation,
-                                              nbins=nbins,
-                                              group_labels=group_labels,
+                                              nbins=nbins, verbose=verbose,
                                               calc_ranges=calc_ranges,
                                               value_range=value_range,
                                               quick=quick, chroms=chroms)
     else:
-        histogram = None
+        histogram = SignalHistogram.read(dirpath, verbose=verbose)
 
     if not replot:
-        histogram.save(dirpath, clobber=clobber)
+        histogram.save(dirpath, clobber=clobber, verbose=verbose)
 
         stats = calc_stats(histogram)
-        save_stats_tab(stats, dirpath, clobber=clobber,
-                       basename=NAMEBASE_STATS)
+        save_stats_tab(stats, dirpath, clobber=clobber, verbose=verbose,
+                       namebase=NAMEBASE_STATS)
 
-        if mnemonicfilename is None and create_mnemonics:
+        if mnemonic_file is None and create_mnemonics:
             statsfilename = make_tabfilename(dirpath, NAMEBASE_STATS)
-            mnemonicfilename = create_mnemonic_file(statsfilename, dirpath,
-                                                    clobber=clobber)
-
-    mnemonics = map_mnemonics(segmentation.labels, mnemonicfilename)
+            mnemonic_file = create_mnemonic_file(statsfilename, dirpath,
+                                                 clobber=clobber,
+                                                 verbose=verbose)
 
     if not noplot:
-        save_stats_plot(dirpath, basename=NAMEBASE_STATS,
-                        clobber=clobber, mnemonics=mnemonics)
-        #save_plot(ntracks, nlabels, segtracks, dirpath, clobber=clobber,
-        #          group_labels=group_labels, ecdf=ecdf, mnemonics=mnemonics)
+        save_stats_plot(dirpath, namebase=NAMEBASE_STATS, clobber=clobber,
+                        mnemonic_file=mnemonic_file, verbose=verbose)
 
     if histogram:
         try:
@@ -863,7 +719,7 @@ def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
 def parse_options(args):
     from optparse import OptionParser, OptionGroup
 
-    usage = "%prog [OPTIONS] BEDFILE GENOMEDATADIR"
+    usage = "%prog [OPTIONS] SEGMENTATION GENOMEDATAFILE"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
 
@@ -872,6 +728,9 @@ def parse_options(args):
                      dest="clobber", default=False,
                      help="Overwrite existing output files if the specified"
                      " directory already exists.")
+    group.add_option("-q", "--quiet", action="store_false",
+                     dest="verbose", default=True,
+                     help="Do not print diagnostic messages.")
     group.add_option("--quick", action="store_true",
                      dest="quick", default=False,
                      help="Compute values only for one chromosome.")
@@ -886,10 +745,6 @@ def parse_options(args):
                      dest="create_mnemonics", default=False,
                      help="If mnemonics are not specified, they will be"
                      " created and used for plotting")
-#     group.add_option("--group-labels", action="store_true",
-#                      dest="group_labels", default=False,
-#                      help="Group track distributions over all labels."
-#                      " BEDFILE will be ignored")
     group.add_option("--ecdf", action="store_true",
                      dest="ecdf", default=False,
                      help="Plot empiracle cumulative density inside each panel"
@@ -925,7 +780,7 @@ def parse_options(args):
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "I/O options")
-    group.add_option("--mnemonic-file", dest="mnemonicfilename",
+    group.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
                      default=None, metavar="FILE",
                      help="If specified, labels will be shown using"
                      " mnemonics found in FILE")
@@ -944,17 +799,14 @@ def parse_options(args):
 
     (options, args) = parser.parse_args(args)
 
-    if len(args) < 2:
-        parser.error("Insufficient number of arguments")
+    if len(args) != 2:
+        parser.error("Inappropriate number of arguments")
 
     if options.inputdirs:
         for inputdir in options.inputdirs:
             if inputdir == options.outdir:
                 parser.error("Output directory cannot be an input directory")
 
-    if options.noplot and options.replot:
-        parser.error("noplot and replot are contradictory")
-
     return (options, args)
 
 ## Command-line entry point
@@ -963,7 +815,7 @@ def main(args=sys.argv[1:]):
     bedfilename = args[0]
     genomedatadir = args[1]
     kwargs = {"clobber": options.clobber,
-#               "group_labels": options.group_labels,
+              "verbose": options.verbose,
               "quick": options.quick,
               "calc_ranges": options.calc_ranges,
               "replot": options.replot,
@@ -972,11 +824,11 @@ def main(args=sys.argv[1:]):
 #               "value_range": (options.min_value, options.max_value),
               "ecdf": options.ecdf,
               "inputdirs": options.inputdirs,
-              "mnemonicfilename": options.mnemonicfilename,
+              "mnemonic_file": options.mnemonic_file,
               "create_mnemonics": options.create_mnemonics,
               "chroms": options.chroms}
 
-    validate(bedfilename, genomedatadir, options.outdir, **kwargs )
+    validate(bedfilename, genomedatadir, options.outdir, **kwargs)
 
 if __name__ == "__main__":
     sys.exit(main())
